#  SPDX-License-Identifier: Apache-2.0
"""
Python Package for controlling Porsche Connect API.

"""
import asyncio
import calendar
import datetime
import json
import logging
import time
import base64
import hashlib
import os
import re
import urllib.parse
from typing import Dict, Text

import aiohttp
from yarl import URL

try:
    from rich import print
except ImportError as e:
    pass

from .exceptions import WrongCredentials, PorscheException

_LOGGER = logging.getLogger(__name__)

async def on_request_start(
        session, trace_config_ctx, params):
    _LOGGER.debug("Starting request")
    _LOGGER.debug(params)

async def on_request_end(session, trace_config_ctx, params):
    _LOGGER.debug("Ending request")
    _LOGGER.debug(params)

trace_config = aiohttp.TraceConfig()
trace_config.on_request_start.append(on_request_start)
trace_config.on_request_end.append(on_request_end)

class Connection:
    """Connection to Porsche Connect API."""

    def __init__(
        self,
        email: Text = None,
        password: Text = None,
        websession: aiohttp.ClientSession = None,
        language: Text = 'de',
        country: Text = 'DE',
        tokens = None
    ) -> None:
        """Initialize connection object."""
        self.porscheCookiedomain: Text = "https://login.porsche.com"
        self.porscheLogin: Text = "https://login.porsche.com/auth/de/de_DE"
        self.porscheLoginAuth: Text = "https://login.porsche.com/auth/api/v1/de/de_DE/public/login"
        self.porscheAPIAuth: Text = "https://login.porsche.com/as/authorization.oauth2"
        self.porscheAPIToken: Text = "https://login.porsche.com/as/token.oauth2"
        self.porscheAPI: Text = "https://connect-portal.porsche.com/core/api/v3/de/de_DE"
        self.porscheApplications = {
                'portal': {
#                    'client_id': '4mPO3OE5Srjb1iaUGWsbqKBvvesya8oA',
                    'client_id': 'TZ4Vf5wnKeipJxvatJ60lPHYEzqZ4WNp',
                    'redirect_uri': 'https://my-static02.porsche.com/static/cms/auth.html',
                    'prefix': 'https://api.porsche.com/core/api/v3/'
                    },
                'carcontrol': {
                    'client_id': 'Ux8WmyzsOAGGmvmWnW7GLEjIILHEztAs',
                    #'client_id': 'imeVsv86TpjM9beWJqI0sNMAip6ga05O',
                    'redirect_uri':  'https://my.porsche.com/myservices/auth/auth.html',
                    'prefix': 'https://api.porsche.com/'
                    }
                }

        self.isTokenRefreshed = False
        self.tokens = tokens or {}
        self.email = email
        self.password = password
        self.websession = websession
        self._isLoggedIn = False
        self.country = country
        self.language = language

        if self.websession == None:
            self.websession = aiohttp.ClientSession(trace_configs=[trace_config])
        _LOGGER.debug("New connection created")


    async def _login(self):
        _LOGGER.debug("Start authentication, get initial state from login page....")
        login_data = { 'sec': '', 'resume': '', 'thirdPartyId': '', 'state': '', 'username':
                self.email, 'password': self.password, 'keeploggedin': 'false' }

        _LOGGER.debug("POST authentication details....")
        async with self.websession.post(self.porscheLoginAuth,  data=login_data, max_redirects=30) as resp:
            # In case of wrong credentials there is a state param in the redirect url
            last_location = resp.history[len(resp.history) - 1].headers['Location']
            query = urllib.parse.urlparse(last_location).query
            redirect_params = urllib.parse.parse_qs(query)
            if "state" in redirect_params and redirect_params["state"][0] == "WRONG_CREDENTIALS":
                raise WrongCredentials("Wrong email or password")

        self._isLoggedIn = True
        return True


    async def getAllTokens(self):
        now = calendar.timegm(datetime.datetime.now().timetuple())
        for applicationKey in self.porscheApplications:
            application = self.porscheApplications[applicationKey]
            token = self.tokens.get(application['client_id'], None)
            if token is None or token['expiration'] < now:
                token = await self._requestToken(application)
                self.tokens[application['client_id']] = token
        self.isTokenRefreshed = False
        return self.tokens

    async def _requestToken(self, application: Dict, wasExpired=False):
        if not self._isLoggedIn or wasExpired:
            await self._login()

        _LOGGER.debug("Requesting access token for client id %s", application['client_id'])

        code_verifier = base64.urlsafe_b64encode(os.urandom(40)).decode('utf-8')
        code_verifier = re.sub('[^a-zA-Z0-9]+', '', code_verifier)

        code_challenge = hashlib.sha256(code_verifier.encode('utf-8')).digest()
        code_challenge = base64.urlsafe_b64encode(code_challenge).decode('utf-8')
        code_challenge = code_challenge.replace('=', '')

        auth_data = {
                'scope': 'openid',
                'response_type': 'code',
                'access_type': 'offline',
                'prompt': 'none',
                'client_id': application['client_id'],
                'redirect_uri': application['redirect_uri'],
                'code_challenge': code_challenge,
                'code_challenge_method': 'S256'
                }

        async with self.websession.get(self.porscheAPIAuth, params=auth_data) as resp:
            last_location = resp.history[len(resp.history) - 1].headers['Location']
            query = urllib.parse.urlparse(last_location).query
            redirect_params = urllib.parse.parse_qs(query)
            auth_code = redirect_params['code'][0]
            _LOGGER.debug("Code %s", auth_code)

        auth_token_data = {
                'grant_type': 'authorization_code',
                'client_id': application['client_id'],
                'redirect_uri': application['redirect_uri'],
                'code': auth_code,
                'prompt': 'none',
                'code_verifier': code_verifier
                }
        _LOGGER.debug("Data %s", auth_token_data)

        now = calendar.timegm(datetime.datetime.now().timetuple())
        async with self.websession.post(self.porscheAPIToken, data=auth_token_data) as resp:
            token_data = await resp.json()
            jwt = self.jwt_payload_decode(token_data['id_token'])
            token = token_data
            token['expiration'] = now + token_data['expires_in']
            token['decoded_token'] = jwt
            token['apikey'] = jwt['aud']
            _LOGGER.debug('Token: %s', token)
            self.isTokenRefreshed = True
            return token

    async def get(self, url, params = None):
        try:
            application = self._applicationForURL(url)
            headers = await self._createhead(application)
            async with self.websession.get(url, params=params, headers=headers) as resp:
                return await resp.json()
        except aiohttp.ClientResponseError as exception_:
            raise PorscheException(exception_.status)

    async def post(self, url, data=None, json=None):
        try:
            application = self._applicationForURL(url)
            headers = await self._createhead(application)
            async with self.websession.post(url, data=data, json=json, headers=headers) as resp:
                return await resp.json()
        except aiohttp.ClientResponseError as exception_:
            raise PorscheException(exception_.status)

    async def put(self, url, data=None, json=None):
        try:
            application = self._applicationForURL(url)
            headers = await self._createhead(application)
            async with self.websession.put(url, data=data, json=json, headers=headers) as resp:
                return await resp.json()
        except aiohttp.ClientResponseError as exception_:
            raise PorscheException(exception_.status)

    async def delete(self, url, data=None, json=None):
        try:
            application = self._applicationForURL(url)
            headers = await self._createhead(application)
            async with self.websession.delete(url, data=data, json=json, headers=headers) as resp:
                return await resp.json()
        except aiohttp.ClientResponseError as exception_:
            raise PorscheException(exception_.status)

    def _applicationForURL(self, url):
       for key in self.porscheApplications:
           app = self.porscheApplications[key]
           if url.startswith(app['prefix']):
               return app
       # else return None
       return None

    async def _createhead(self, application):
        # If no application matched the request URL then no auth headers are added
        if application is None:
            return {}
        now = calendar.timegm(datetime.datetime.now().timetuple())
        token = self.tokens.get(application['client_id'], None)
        if token is None or token['expiration'] < now:
            token = await self._requestToken(application, wasExpired=(token is not None))
            self.tokens[application['client_id']] = token
        head = {
            "Authorization": f"Bearer {token['access_token']}",
            "apikey": token['apikey'],
            "x-vrs-url-country": self.country.lower(),
            "x-vrs-url-language": f"{self.language.lower()}_{self.country.upper()}"
            }
        return head

    def _b64_decode(self, data):
        data += '=' * (4 - len(data) % 4)
        return base64.b64decode(data).decode('utf-8')

    def jwt_payload_decode(self, jwt):
        _, payload, _ = jwt.split('.')
        return json.loads(self._b64_decode(payload))

    async def close(self):
        await self.websession.close()




from pyporscheconnectapi.connection import Connection
import asyncio
import datetime

class Client:
    """Client for Porsche Connect API."""

    def __init__(
        self,
        connection: Connection
    ) -> None:
        self._connection = connection


    def isTokenRefreshed(self):
        return self._connection.isTokenRefreshed

    async def getAllTokens(self):
        return await self._connection.getAllTokens()

    async def _spinner(self, url):
        while True:
            data = await self._connection.get(url)
            if 'status' in data and data['status'] != 'IN_PROGRESS':
                break
            elif 'actionState' in data and data['actionState'] != 'IN_PROGRESS':
                break
            await asyncio.sleep(1)
        return data

    async def _lockUnlock(self, vin, pin, action, waitForConfirmation=True):
        progressResult = await self._connection.post(f"https://api.porsche.com/service-vehicle/remote-lock-unlock/{vin}/{action}", json={ 'pin': pin })
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/service-vehicle/remote-lock-unlock/{vin}/{progressResult['requestId']}/status")
        if(result['status'] == 'SUCCESS'):
            result = await self._connection.get(f"https://api.porsche.com/service-vehicle/remote-lock-unlock/{vin}/last-actions")
        return result

    async def _setClimate(self, vin, action, country = 'de', language = 'de', waitForConfirmation=True):
        progressResult = await self._connection.post(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/{vin}/toggle-direct-climatisation/{action}", json={})
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/e-mobility/se/sv_SE/{vin}/toggle-direct-climatisation/status/{progressResult['requestId']}")
        return result

    async def _setDirectCharge(self, vin, action, model=None, country = 'de', language = 'de', waitForConfirmation=True):
        if model is None:
            data = await self.getCapabilities(vin)
            model = data['carModel']
        progressResult = await self._connection.post(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/{model}/{vin}/toggle-direct-charging/{action}", json={})
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/e-mobility/se/sv_SE/{model}/{vin}/toggle-direct-charging/status/{progressResult['requestId']}")
        return result

    async def _addTimer(self, vin, timer, country = 'de', language = 'de', waitForConfirmation=True):
        """Add new charge & climate timer"""
        progressResult = await self._connection.post(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/timer", json=timer)
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/action-status/{progressResult['actionId']}")
        return result

    async def _updateTimer(self, vin, timer, timerID='1', country = 'de', language = 'de', waitForConfirmation=True):
        """Update existing charge & climate timer"""
        timer.update({"timerID":timerID})
        progressResult = await self._connection.put(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/timer", json=timer)
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/action-status/{progressResult['actionId']}")
        return result

    async def _deleteTimer(self, vin, timerID='1', country = 'de', language = 'de', waitForConfirmation=True):
        """Delete existing charge & climate timer"""
        progressResult = await self._connection.delete(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/timer/{timerID}")
        if not waitForConfirmation: return progressResult
        result = await self._spinner(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/J1/{vin}/action-status/{progressResult['actionId']}")
        return result

    def _formatTimer(self, active, charge_settings, climate, time_date):
        """Format combined payload for charge & climate timer"""
        payload = {"active": active, "climatised": climate}
        payload.update(charge_settings)
        payload.update(time_date)
        return payload

    def _formatChargeTimer(self, charge=False, target_charge=10):
        """Format payload for charge timer
        Parameters:
        charge (bool): Enable charging?
        target_charge (0-100): Target charge level
        """
        charge_level = min(max(int(target_charge),0),100)
        return {"chargeOption": charge, "targetChargeLevel": charge_level}

    def _formatTimerTime(self, time, repeat_days=[]):
        """Format time/date settings for timer
        Parameters:
        time (datetime): Target departure date/time as datetime object
        repeat_days (list of integers): Which days to repeat on, if any, 0=Monday)
        """

        #FIXME: There's something missing here for creation of repeating timers
        # Hard to debug b/c the website also isn't letting me right now (app is fine)
        # Format seems to work for updating timers
        if len(repeat_days):
            frequency = "CYCLIC"
        else:
            frequency = "SINGLE"
        repeat_dict = {"MONDAY": False, "TUESDAY": False, "WEDNESDAY": False, "THURSDAY": False, "FRIDAY": False,\
                "SATURDAY": False, "SUNDAY": False}
        for i in repeat_days:
            if i==0: repeat_dict["MONDAY"] = True
            elif i==1: repeat_dict["TUESDAY"] = True
            elif i==2: repeat_dict["WEDNESDAY"] = True
            elif i==3: repeat_dict["THURSDAY"] = True
            elif i==4: repeat_dict["FRIDAY"] = True
            elif i==5: repeat_dict["SATURDAY"] = True
            elif i==6: repeat_dict["SUNDAY"] = True

        return {"departureDateTime": time.isoformat(), "frequency": frequency, "weekDays": repeat_dict}

    async def newTimer(self, vin, time, active=False, charge=False, target_charge=10,\
            climate=False, repeat_days=[], waitForConfirmation=True):
        """Create a new timer on the vehicle
        Parameters:
        vin (string): Vehicle VIN
        time (datetime): Target departure date/time as datetime object
        active (bool): Timer should be active upon creation?
        charge (bool): Enable charging?
        target_charge (0-100): Target charge level
        climate (bool): Precondition cabin?
        repeat_days (list of integers): Which days to repeat on, if any, 0=Monday)

        FIXME: Creation of repeating timers isn't working, see _formatTimerTime
        """
        timer_time = self._formatTimerTime(time, repeat_days)
        charge_timer = self._formatChargeTimer(charge, target_charge)
        full_timer = self._formatTimer(active, charge_timer, climate, timer_time)
        return await self._addTimer(vin, full_timer, waitForConfirmation=waitForConfirmation)

    async def updateTimer(self, vin, time, timerID='1', active=False, charge=False, target_charge=10,\
            climate=False, repeat_days=[], waitForConfirmation=True):
        """Update an existing timer on the vehicle
        Parameters:
        vin (string): Vehicle VIN
        time (datetime): Target departure date/time as datetime object
        timerId (string): Target timer (numeric string)
        active (bool): Timer should be active upon creation?
        charge (bool): Enable charging?
        target_charge (0-100): Target charge level
        climate (bool): Precondition cabin?
        repeat_days (list of integers): Which days to repeat on, if any, 0=Monday)
        """
        timer_time = self._formatTimerTime(time, repeat_days)
        charge_timer = self._formatChargeTimer(charge, target_charge)
        full_timer = self._formatTimer(active, charge_timer, climate, timer_time)
        return await self._updateTimer(vin, full_timer, timerID, waitForConfirmation=waitForConfirmation)

    async def deleteTimer(self, vin, timerID='1', waitForConfirmation=True):
        """Delete an existing timer on the vehicle
        Parameters:
        vin (string): Vehicle VIN
        timerId (string): Target timer (numeric string)
        """
        return await self._deleteTimer(vin, timerID, waitForConfirmation=waitForConfirmation)

    async def lock(self, vin, pin, waitForConfirmation=True):
        return await self._lockUnlock(vin, pin, 'lock', waitForConfirmation=waitForConfirmation)

    async def unlock(self, vin, pin, waitForConfirmation=True):
        return await self._lockUnlock(vin, pin, 'unlock', waitForConfirmation=waitForConfirmation)

    async def climateOn(self, vin, waitForConfirmation=True):
        return await self._setClimate(vin, 'true', waitForConfirmation=waitForConfirmation)

    async def climateOff(self, vin, waitForConfirmation=True):
        return await self._setClimate(vin,'false', waitForConfirmation=waitForConfirmation)

    async def directChargeOn(self, vin, model, waitForConfirmation=True):
        return await self._setDirectCharge(vin, 'true', model, waitForConfirmation=waitForConfirmation)

    async def directChargeOff(self, vin, model, waitForConfirmation=True):
        return await self._setDirectCharge(vin, 'false', model, waitForConfirmation=waitForConfirmation)

    async def getVehicles(self):
        vehicles = await self._connection.get("https://api.porsche.com/core/api/v3/se/sv_SE/vehicles")
        return vehicles

    async def getStoredOverview(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/se/sv_SE/vehicle-data/{vin}/stored")
        return data

    async def getCurrentOverview(self, vin):
        progressResult = await self._connection.post(f"https://api.porsche.com/service-vehicle/se/sv_SE/vehicle-data/{vin}/current/request")
        spinnerResult = await self._spinner(f"https://api.porsche.com/service-vehicle/se/sv_SE/vehicle-data/{vin}/current/request/{progressResult['requestId']}/status")
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/se/sv_SE/vehicle-data/{vin}/current/request/{progressResult['requestId']}")
        return data

    async def getMaintenance(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/predictive-maintenance/information/{vin}")
        return data

    async def getSummary(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/vehicle-summary/{vin}")
        return data

    async def getCapabilities(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/vcs/capabilities/{vin}")
        return data

    async def getPosition(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/car-finder/{vin}/position")
        return data

    async def getTheftAlerts(self, vin):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/theft-alerts/{vin}/history")
        return data

    async def getSpeedAlerts(self, vin, country= 'DE', language = 'de'):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/{country.lower()}/{language.lower()}_{country.upper()}/speed-alert/{vin}/alerts")
        return data

    async def getTripLongTerm(self, vin, country= 'DE', language = 'de'):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/{country.lower()}/{language.lower()}_{country.upper()}/trips/{vin}/LONG_TERM/newest")
        return data

    async def getTripShortTerm(self, vin, country= 'DE', language = 'de'):
        data = await self._connection.get(f"https://api.porsche.com/service-vehicle/{country.lower()}/{language.lower()}_{country.upper()}/trips/{vin}/SHORT_TERM")
        return data

    async def getEmobility(self, vin, model = None, country= 'DE', language = 'de', timezone='Europe/Stockholm'):
        if model is None:
            data = await self.getCapabilities(vin)
            model = data['carModel']
        data = await self._connection.get(f"https://api.porsche.com/e-mobility/{country.lower()}/{language.lower()}_{country.upper()}/{model}/{vin}?timezone={timezone}")
        return data









import argparse
import asyncio
import configparser
from pyporscheconnectapi.connection import Connection
from pyporscheconnectapi.client import Client
from pyporscheconnectapi.exceptions import WrongCredentials
import os
import sys
import logging
import json
from getpass import getpass
try:
    from rich import print
except ImportError as e:
    pass

logging.basicConfig()
logging.root.setLevel(logging.WARNING)


async def main(args):
    try:
        with open(args.session_file) as json_file:
            tokens = json.load(json_file)
    except FileNotFoundError:
        tokens = {}
    except json.decoder.JSONDecodeError:
        tokens = {}

    email = args.email or input("Please enter Porsche Connect email: ")
    password = args.password or getpass()
    conn = Connection(email, password, tokens=tokens, country=args.country, language=args.language)

    client = Client(conn)

    try:
        if args.command == "list":
            data = await client.getVehicles()
            print(json.dumps(data, indent=2))
        elif args.command == "tokens":
            data = await client.getAllTokens()
            print(json.dumps(data, indent=2))
        else:
            vins = []
            if args.vin is not None: vins = [ args.vin ]
            elif args.all:
                vehicles = await client.getVehicles()
                vins = map(lambda v : v['vin'], vehicles)
            else:
                sys.exit("--vin or --all is required")
            for vin in vins:
                data = {}
                if args.command == "overview":
                    if(args.nowait):
                        data = await client.getStoredOverview(vin)
                    else:
                        data = await client.getCurrentOverview(vin)
                elif args.command == "maintenance":
                    data = await client.getMaintenance(vin)
                elif args.command == "summary":
                    data = await client.getSummary(vin)
                elif args.command == "capabilities":
                    data = await client.getCapabilities(vin)
                elif args.command == "position":
                    data = await client.getPosition(vin)
                elif args.command == "emobility":
                    data = await client.getEmobility(vin, model=args.model, country=args.country, language=args.language, timezone=args.timezone)
                elif args.command == "triplongterm":
                    data = await client.getTripLongTerm(vin, country=args.country, language=args.language)
                elif args.command == "tripshortterm":
                    data = await client.getTripShortTerm(vin, country=args.country, language=args.language)
                elif args.command == "speedalerts":
                    data = await client.getSpeedAlerts(vin, country=args.country, language=args.language)
                elif args.command == "theftalerts":
                    data = await client.getTheftAlerts(vin)
                elif args.command == "climate-on":
                    data = await client.climateOn(vin, waitForConfirmation = not args.nowait)
                elif args.command == "climate-off":
                    data = await client.climateOff(vin, waitForConfirmation = not args.nowait)
                elif args.command == "directcharge-on":
                    data = await client.directChargeOn(vin, model=args.model, waitForConfirmation = not args.nowait)
                elif args.command == "directcharge-off":
                    data = await client.directChargeOff(vin, model=args.model, waitForConfirmation = not args.nowait)
                elif args.command == 'lock' or args.command == 'unlock':
                    pin = args.pin
                    if pin is None:
                        pin = getpass("PIN code: ")
                    if(args.command == 'lock'):
                        data = await client.lock(vin, pin, waitForConfirmation = not args.nowait)
                    else:
                        data = await client.unlock(vin, pin, waitForConfirmation = not args.nowait)

                print(json.dumps(data, indent=2))
    except WrongCredentials:
        sys.exit("Wrong email or password")

    await conn.close()
    with open(args.session_file, 'w', encoding='utf-8') as json_file:
        json.dump(conn.tokens, json_file, ensure_ascii=False, indent=2)

def cli():
    config = configparser.ConfigParser()
    config["porsche"] = {"email": "", "password": "", "session_file": ".session", "country": "DE", "language": "de",
            "timezone": "Europe/Stockholm"}
    config.read([".porscheconnect.cfg", os.path.expanduser("~/.porscheconnect.cfg")])
    parser = argparse.ArgumentParser(description='Porsche Connect CLI.')
    parser.add_argument('command', choices=['list', 'overview', 'maintenance', 'summary', 'capabilities', 'emobility',
        'position', 'triplongterm', 'tripshortterm', 'speedalerts', 'theftalerts', 'tokens', 'lock', 'unlock',
        'climate-on', 'climate-off', 'directcharge-on', 'directcharge-off'])
    parser.add_argument('-e', '--email', dest='email', default=config.get('porsche', 'email'))
    parser.add_argument('-p', '--password', dest='password', default=config.get('porsche', 'password'))
    parser.add_argument('-s', '--sessionfile', dest='session_file', default=config.get('porsche', 'session_file'))
    parser.add_argument('-v', '--vin', dest='vin', default=None)
    parser.add_argument('-n', '--pin', dest='pin', default=None)
    parser.add_argument('-m', '--model', dest='model', default=None)
    parser.add_argument('-a', '--all', dest='all', action='store_true')
    parser.add_argument('-c', '--country', dest='country', default=config.get('porsche', 'country'))
    parser.add_argument('-l', '--language', dest='language', default=config.get('porsche', 'language'))
    parser.add_argument('-z', '--timezone', dest='timezone', default=config.get('porsche', 'timezone'))
    parser.add_argument('--nowait', dest='nowait', action='store_true')

    args = parser.parse_args()

    loop = asyncio.get_event_loop()
    loop.run_until_complete(main(args))
